@@ -1,10 +1,10 @@
 
 /* Compiler implementation of the D programming language
- * Copyright (C) 1999-2021 by The D Language Foundation, All Rights Reserved
+ * Copyright (C) 1999-2022 by The D Language Foundation, All Rights Reserved
  * written by Walter Bright
- * http://www.digitalmars.com
+ * https://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
- * http://www.boost.org/LICENSE_1_0.txt
+ * https://www.boost.org/LICENSE_1_0.txt
  * https://github.com/dlang/dmd/blob/master/src/dmd/aggregate.h
  */
 
@@ -105,11 +105,12 @@ public:
 
     AliasThis *aliasthis;       // forward unresolved lookups to aliasthis
 
-    DtorDeclarations dtors;     // Array of destructors
-    DtorDeclaration *dtor;      // aggregate destructor
-    DtorDeclaration *primaryDtor; // non-deleting C++ destructor, same as dtor for D
+    DtorDeclarations userDtors; // user-defined destructors (`~this()`) - mixins can yield multiple ones
+    DtorDeclaration *aggrDtor;  // aggregate destructor calling userDtors and fieldDtor (and base class aggregate dtor for C++ classes)
+    DtorDeclaration *dtor;      // the aggregate destructor exposed as `__xdtor` alias
+                                // (same as aggrDtor, except for C++ classes with virtual dtor on Windows)
     DtorDeclaration *tidtor;    // aggregate destructor used in TypeInfo (must have extern(D) ABI)
-    FuncDeclaration *fieldDtor;   // aggregate destructor for just the fields
+    DtorDeclaration *fieldDtor; // function destructing (non-inherited) fields
 
     Expression *getRTInfo;      // pointer to GC info generated by object.RTInfo(this)
 
@@ -121,10 +122,10 @@ public:
     virtual Scope *newScope(Scope *sc);
     void setScope(Scope *sc);
     size_t nonHiddenFields();
-    bool determineSize(Loc loc);
+    bool determineSize(const Loc &loc);
     virtual void finalizeSize() = 0;
     d_uns64 size(const Loc &loc);
-    bool fill(Loc loc, Expressions *elements, bool ctorinit);
+    bool fill(const Loc &loc, Expressions *elements, bool ctorinit);
     Type *getType();
     bool isDeprecated() const;         // is aggregate deprecated?
     void setDeprecated();
@@ -188,7 +189,7 @@ public:
     // ABI-specific type(s) if the struct can be passed in registers
     TypeTuple *argTypes;
 
-    static StructDeclaration *create(Loc loc, Identifier *id, bool inObject);
+    static StructDeclaration *create(const Loc &loc, Identifier *id, bool inObject);
     StructDeclaration *syntaxCopy(Dsymbol *s);
     Dsymbol *search(const Loc &loc, Identifier *ident, int flags = SearchLocalsOnly);
     const char *kind() const;
@@ -283,7 +284,7 @@ public:
     Symbol *cpp_type_info_ptr_sym;      // cached instance of class Id.cpp_type_info_ptr
 #endif
 
-    static ClassDeclaration *create(Loc loc, Identifier *id, BaseClasses *baseclasses, Dsymbols *members, bool inObject);
+    static ClassDeclaration *create(const Loc &loc, Identifier *id, BaseClasses *baseclasses, Dsymbols *members, bool inObject);
     const char *toPrettyChars(bool QualifyTypes = false);
     ClassDeclaration *syntaxCopy(Dsymbol *s);
     Scope *newScope(Scope *sc);
