@@ -1,10 +1,10 @@
 
 /* Compiler implementation of the D programming language
- * Copyright (C) 1999-2021 by The D Language Foundation, All Rights Reserved
+ * Copyright (C) 1999-2022 by The D Language Foundation, All Rights Reserved
  * written by Walter Bright
- * http://www.digitalmars.com
+ * https://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
- * http://www.boost.org/LICENSE_1_0.txt
+ * https://www.boost.org/LICENSE_1_0.txt
  * https://github.com/dlang/dmd/blob/master/src/dmd/template.h
  */
 
@@ -135,7 +135,7 @@ public:
     virtual bool declareParameter(Scope *sc) = 0;
     virtual void print(RootObject *oarg, RootObject *oded) = 0;
     virtual RootObject *specialization() = 0;
-    virtual RootObject *defaultArg(Loc instLoc, Scope *sc) = 0;
+    virtual RootObject *defaultArg(const Loc &instLoc, Scope *sc) = 0;
     virtual bool hasDefaultArg() = 0;
 
     /* Create dummy argument based on parameter.
@@ -158,7 +158,7 @@ public:
     bool declareParameter(Scope *sc);
     void print(RootObject *oarg, RootObject *oded);
     RootObject *specialization();
-    RootObject *defaultArg(Loc instLoc, Scope *sc);
+    RootObject *defaultArg(const Loc &instLoc, Scope *sc);
     bool hasDefaultArg();
     RootObject *dummyArg();
     void accept(Visitor *v) { v->visit(this); }
@@ -190,7 +190,7 @@ public:
     bool declareParameter(Scope *sc);
     void print(RootObject *oarg, RootObject *oded);
     RootObject *specialization();
-    RootObject *defaultArg(Loc instLoc, Scope *sc);
+    RootObject *defaultArg(const Loc &instLoc, Scope *sc);
     bool hasDefaultArg();
     RootObject *dummyArg();
     void accept(Visitor *v) { v->visit(this); }
@@ -211,7 +211,7 @@ public:
     bool declareParameter(Scope *sc);
     void print(RootObject *oarg, RootObject *oded);
     RootObject *specialization();
-    RootObject *defaultArg(Loc instLoc, Scope *sc);
+    RootObject *defaultArg(const Loc &instLoc, Scope *sc);
     bool hasDefaultArg();
     RootObject *dummyArg();
     void accept(Visitor *v) { v->visit(this); }
@@ -228,7 +228,7 @@ public:
     bool declareParameter(Scope *sc);
     void print(RootObject *oarg, RootObject *oded);
     RootObject *specialization();
-    RootObject *defaultArg(Loc instLoc, Scope *sc);
+    RootObject *defaultArg(const Loc &instLoc, Scope *sc);
     bool hasDefaultArg();
     RootObject *dummyArg();
     void accept(Visitor *v) { v->visit(this); }
@@ -288,6 +288,7 @@ public:
     Identifier *getIdent();
     hash_t toHash();
 
+    bool isDiscardable();
     bool needsCodegen();
 
     TemplateInstance *isTemplateInstance() { return this; }
