@@ -353,9 +353,10 @@ static cl::list<std::string, StringsAdapter> stringImportPaths(
     "J", cl::desc("Look for string imports also in <directory>"),
     cl::value_desc("directory"), cl::location(strImpPathStore), cl::Prefix);
 
-static cl::opt<bool, true>
-    addMain("main", cl::desc("Add default main() (e.g. for unittesting)"),
-            cl::ZeroOrMore, cl::location(global.params.addMain));
+static cl::opt<bool, true> addMain(
+    "main", cl::ZeroOrMore, cl::location(global.params.addMain),
+    cl::desc(
+        "Add default main() if not present already (e.g. for unittesting)"));
 
 // -d-debug is a bit messy, it has 3 modes:
 // -d-debug=ident, -d-debug=level and -d-debug (without argument)
@@ -514,11 +515,16 @@ cl::opt<uint32_t, true> hashThreshold(
     "hash-threshold", cl::ZeroOrMore, cl::location(global.params.hashThreshold),
     cl::desc("Hash symbol names longer than this threshold (experimental)"));
 
-static cl::opt<bool, true> linkonceTemplates(
-    "linkonce-templates", cl::ZeroOrMore,
-    cl::location(global.params.linkonceTemplates),
-    cl::desc(
-        "Use linkonce_odr linkage for template symbols instead of weak_odr"));
+static cl::opt<LinkonceTemplates, true> linkonceTemplates(
+    cl::ZeroOrMore, cl::location(global.params.linkonceTemplates),
+    cl::values(
+        clEnumValN(LinkonceTemplates::yes, "linkonce-templates",
+                   "Use discardable linkonce_odr linkage for template symbols "
+                   "and lazily & recursively define all referenced "
+                   "instantiated symbols in each object file"),
+        clEnumValN(LinkonceTemplates::aggressive,
+                   "linkonce-templates-aggressive",
+                   "Experimental, more aggressive variant")));
 
 cl::opt<bool> disableLinkerStripDead(
     "disable-linker-strip-dead", cl::ZeroOrMore,
@@ -731,7 +737,7 @@ void hideLLVMOptions() {
   static const char *const hiddenOptions[] = {
       "aarch64-neon-syntax", "aarch64-use-aa",
       "abort-on-max-devirt-iterations-reached",
-      "addrsig", "allow-ginsert-as-artifact",
+      "addrsig", "align-loops", "allow-ginsert-as-artifact",
       "amdgpu-bypass-slow-div", "amdgpu-disable-loop-alignment",
       "amdgpu-disable-power-sched", "amdgpu-dpp-combine",
       "amdgpu-dump-hsa-metadata", "amdgpu-enable-flat-scratch",
@@ -750,6 +756,7 @@ void hideLLVMOptions() {
       "cvp-dont-add-nowrap-flags",
       "cvp-dont-process-adds", "debug-counter", "debug-entry-values",
       "debugger-tune", "debugify-level", "debugify-quiet",
+      "debug-info-correlate",
       "denormal-fp-math", "denormal-fp-math-f32", "disable-debug-info-verifier",
       "disable-i2p-p2i-opt",
       "disable-objc-arc-checkforcfghazards", "disable-promote-alloca-to-lds",
@@ -775,6 +782,8 @@ void hideLLVMOptions() {
       "experimental-debug-variable-locations",
       "fatal-assembler-warnings", "filter-print-funcs",
       "force-dwarf-frame-section", "force-opaque-pointers",
+      "fs-profile-debug-bw-threshold", "fs-profile-debug-prob-diff-threshold",
+      "generate-merged-base-profiles",
       "gpsize", "hash-based-counter-split",
       "hot-cold-split", "ignore-xcoff-visibility",
       "imp-null-check-page-size", "imp-null-max-insts-to-consider",
@@ -788,7 +797,7 @@ void hideLLVMOptions() {
       "join-liveintervals", "jump-table-type", "limit-float-precision",
       "lto-embed-bitcode", "matrix-default-layout", "matrix-propagate-shape",
       "max-counter-promotions", "max-counter-promotions-per-loop",
-      "mc-relax-all", "mc-x86-disable-arith-relaxation", "meabi",
+      "mc-relax-all", "mc-x86-disable-arith-relaxation", "mcabac", "meabi",
       "memop-size-large", "memop-size-range", "merror-missing-parenthesis",
       "merror-noncontigious-register", "mfuture-regs", "mhvx",
       "mips-compact-branches", "mips16-constant-islands", "mips16-hard-float",
@@ -797,11 +806,12 @@ void hideLLVMOptions() {
       "mwarn-noncontigious-register", "mwarn-sign-mismatch",
       "no-discriminators", "no-type-check", "no-xray-index",
       "nozero-initialized-in-bss", "nvptx-sched4reg",
-      "objc-arc-annotation-target-identifier", "pie-copy-relocations",
-      "poison-checking-function-local",
+      "objc-arc-annotation-target-identifier", "opaque-pointers",
+      "pie-copy-relocations", "poison-checking-function-local",
       "polly-dump-after", "polly-dump-after-file", "polly-dump-before",
       "polly-dump-before-file", "pre-RA-sched", "print-after-all",
       "print-before-all", "print-machineinstrs", "print-module-scope",
+      "print-pipeline-passes",
       "profile-estimator-loop-weight", "profile-estimator-loop-weight",
       "profile-file", "profile-info-file", "profile-verifier-noassert",
       "pseudo-probe-for-profiling",
@@ -824,8 +834,8 @@ void hideLLVMOptions() {
       "stack-size-section", "stack-symbol-ordering",
       "stackmap-version", "static-func-full-module-prefix",
       "static-func-strip-dirname-prefix", "stats", "stats-json", "strict-dwarf",
-      "strip-debug", "struct-path-tbaa", "summary-file", "tail-predication",
-      "tailcallopt", "thinlto-assume-merged",
+      "strip-debug", "struct-path-tbaa", "summary-file", "swift-async-fp",
+      "tail-predication", "tailcallopt", "thinlto-assume-merged",
       "thread-model", "time-passes", "time-trace-granularity", "tls-size",
       "unfold-element-atomic-memcpy-max-elements",
       "unique-basic-block-section-names", "unique-bb-section-names",
@@ -835,6 +845,7 @@ void hideLLVMOptions() {
       "verify-loop-lcssa", "verify-machine-dom-info", "verify-regalloc",
       "verify-region-info", "verify-scev", "verify-scev-maps",
       "vp-counters-per-site", "vp-static-alloc",
+      "wasm-enable-eh", "wasm-enable-sjlj",
       "x86-align-branch", "x86-align-branch-boundary",
       "x86-branches-within-32B-boundaries", "x86-early-ifcvt",
       "x86-pad-max-prefix-size",
