@@ -24,6 +24,9 @@
 #include "gen/modules.h"
 #include "gen/runtime.h"
 #include "ir/irdsymbol.h"
+#if LDC_LLVM_VER >= 1400
+#include "llvm/IR/DiagnosticInfo.h"
+#endif
 #if LDC_LLVM_VER >= 1100
 #include "llvm/IR/LLVMRemarkStreamer.h"
 #elif LDC_LLVM_VER >= 900
