@@ -489,6 +489,14 @@ void ArgsBuilder::build(llvm::StringRef outputPath,
     args.push_back(objfile);
   }
 
+  // add precompiled rt.dso object file (in lib directory) when linking
+  // against shared druntime
+  const auto &libDirs = ConfigFile::instance.libDirs();
+  if (!defaultLibNames.empty() && linkAgainstSharedDefaultLibs() &&
+      !libDirs.empty()) {
+    args.push_back((llvm::Twine(libDirs[0]) + "/ldc_rt.dso.o").str());
+  }
+
   // Link with profile-rt library when generating an instrumented binary.
   if (opts::isInstrumentingForPGO()) {
     addProfileRuntimeLinkFlags(*global.params.targetTriple);
@@ -533,7 +541,7 @@ void ArgsBuilder::build(llvm::StringRef outputPath,
   addUserSwitches();
 
   // lib dirs
-  for (const char *dir_c : ConfigFile::instance.libDirs()) {
+  for (const char *dir_c : libDirs) {
     const llvm::StringRef dir(dir_c);
     if (!dir.empty())
       args.push_back(("-L" + dir).str());
@@ -587,22 +595,12 @@ void ArgsBuilder::build(llvm::StringRef outputPath,
 void ArgsBuilder::addLinker() {
   llvm::StringRef linker = opts::linker;
 
-  // We have a default linker preference for Linux targets. It can be disabled
-  // via `-linker=` (explicitly empty).
-  if (global.params.targetTriple->isOSLinux() &&
+  // Default to ld.bfd for Android (placing .tdata and .tbss sections adjacent
+  // to each other as required by druntime's rt.sections_android, contrary to
+  // gold and lld as of Android NDK r21d).
+  if (global.params.targetTriple->getEnvironment() == llvm::Triple::Android &&
       opts::linker.getNumOccurrences() == 0) {
-    // Default to ld.bfd for Android (placing .tdata and .tbss sections adjacent
-    // to each other as required by druntime's rt.sections_android, contrary to
-    // gold and lld as of Android NDK r21d).
-    if (global.params.targetTriple->getEnvironment() == llvm::Triple::Android) {
-      linker = "bfd";
-    }
-    // Otherwise default to ld.gold for Linux due to ld.bfd issues with ThinLTO
-    // (see #2278) and older bfd versions stripping llvm.used symbols (e.g.,
-    // ModuleInfo refs) with --gc-sections (see #2870).
-    else {
-      linker = "gold";
-    }
+    linker = "bfd";
   }
 
   if (!linker.empty())
@@ -752,10 +750,18 @@ int linkObjToBinaryGcc(llvm::StringRef outputPath,
 
     bool success = false;
     if (global.params.targetTriple->isOSBinFormatELF()) {
-      success = lld::elf::link(fullArgs, CanExitEarly
+      success = lld::elf::link(fullArgs
+#if LDC_LLVM_VER < 1400
+                               ,
+                               CanExitEarly
+#endif
 #if LDC_LLVM_VER >= 1000
                                ,
                                llvm::outs(), llvm::errs()
+#endif
+#if LDC_LLVM_VER >= 1400
+                                                 ,
+                               CanExitEarly, false
 #endif
       );
     } else if (global.params.targetTriple->isOSBinFormatMachO()) {
@@ -764,20 +770,32 @@ int linkObjToBinaryGcc(llvm::StringRef outputPath,
 #else
       success = lld::mach_o::link(fullArgs
 #endif
-#if LDC_LLVM_VER >= 700
+#if LDC_LLVM_VER >= 700 && LDC_LLVM_VER < 1400
                                  ,
                                  CanExitEarly
+#endif
 #if LDC_LLVM_VER >= 1000
                                  ,
                                  llvm::outs(), llvm::errs()
 #endif
+#if LDC_LLVM_VER >= 1400
+                                                   ,
+                                 CanExitEarly, false
 #endif
       );
     } else if (global.params.targetTriple->isOSBinFormatCOFF()) {
       success = lld::mingw::link(fullArgs
+#if LDC_LLVM_VER >= 1000 && LDC_LLVM_VER < 1400
+                                 ,
+                                 CanExitEarly
+#endif
 #if LDC_LLVM_VER >= 1000
                                  ,
-                                 CanExitEarly, llvm::outs(), llvm::errs()
+                                 llvm::outs(), llvm::errs()
+#endif
+#if LDC_LLVM_VER >= 1400
+                                                   ,
+                                 CanExitEarly, false
 #endif
       );
     } else if (global.params.targetTriple->isOSBinFormatWasm()) {
@@ -786,10 +804,18 @@ int linkObjToBinaryGcc(llvm::StringRef outputPath,
       //        but only on Linux?
       CanExitEarly = true;
 #endif
-      success = lld::wasm::link(fullArgs, CanExitEarly
+      success = lld::wasm::link(fullArgs
+#if LDC_LLVM_VER < 1400
+                                ,
+                                CanExitEarly
+#endif
 #if LDC_LLVM_VER >= 1000
                                 ,
                                 llvm::outs(), llvm::errs()
+#endif
+#if LDC_LLVM_VER >= 1400
+                                                  ,
+                                CanExitEarly, false
 #endif
       );
     } else {
