@@ -158,7 +158,7 @@ static void DtoArrayInit(const Loc &loc, LLValue *ptr, LLValue *length,
   // assign array element value
   DLValue arrayelem(elementValue->type->toBasetype(),
                     DtoGEP1(ptr, itr_val, "arrayinit.arrayelem"));
-  DtoAssign(loc, &arrayelem, elementValue, TOKblit);
+  DtoAssign(loc, &arrayelem, elementValue, EXP::blit);
 
   // increment iterator
   DtoStore(gIR->ir->CreateAdd(itr_val, DtoConstSize_t(1), "arrayinit.new_itr"),
@@ -222,7 +222,7 @@ static bool arrayNeedsPostblit(Type *t) {
 
 // Does array assignment (or initialization) from another array of the same
 // element type or from an appropriate single element.
-void DtoArrayAssign(const Loc &loc, DValue *lhs, DValue *rhs, int op,
+void DtoArrayAssign(const Loc &loc, DValue *lhs, DValue *rhs, EXP op,
                     bool canSkipPostblit) {
   IF_LOG Logger::println("DtoArrayAssign");
   LOG_SCOPE;
@@ -242,9 +242,9 @@ void DtoArrayAssign(const Loc &loc, DValue *lhs, DValue *rhs, int op,
     return;
   }
 
-  // TOKblit is generated by the frontend for (default) initialization of
+  // EXP::blit is generated by the frontend for (default) initialization of
   // static arrays of structs with a single element.
-  const bool isConstructing = (op == TOKconstruct || op == TOKblit);
+  const bool isConstructing = (op == EXP::construct || op == EXP::blit);
 
   Type *const elemType = t->nextOf()->toBasetype();
   const bool needsDestruction =
@@ -265,7 +265,7 @@ void DtoArrayAssign(const Loc &loc, DValue *lhs, DValue *rhs, int op,
     LLValue *rhsPtr = DtoBitCast(realRhsArrayPtr, getVoidPtrType());
     LLValue *rhsLength = DtoArrayLen(rhs);
 
-    const bool needsPostblit = (op != TOKblit && arrayNeedsPostblit(t) &&
+    const bool needsPostblit = (op != EXP::blit && arrayNeedsPostblit(t) &&
                                 (!canSkipPostblit || t2->ty == TY::Tarray));
 
     if (!needsDestruction && !needsPostblit) {
@@ -309,7 +309,7 @@ void DtoArrayAssign(const Loc &loc, DValue *lhs, DValue *rhs, int op,
     // T[]  = T     T[n][]  = T
     // T[n] = T     T[n][m] = T
     const bool needsPostblit =
-        (op != TOKblit && !canSkipPostblit && arrayNeedsPostblit(t));
+        (op != EXP::blit && !canSkipPostblit && arrayNeedsPostblit(t));
 
     if (!needsDestruction && !needsPostblit) {
       // fast version
@@ -512,7 +512,7 @@ bool isConstLiteral(Expression *e, bool immutableType) {
   // Examine the ArrayLiteralExps and the StructLiteralExps element by element
   // as isConst always returns 0 on those.
   switch (e->op) {
-  case TOKarrayliteral: {
+  case EXP::arrayLiteral: {
     auto ale = static_cast<ArrayLiteralExp *>(e);
 
     if (!immutableType) {
@@ -529,7 +529,7 @@ bool isConstLiteral(Expression *e, bool immutableType) {
     }
   } break;
 
-  case TOKstructliteral: {
+  case EXP::structLiteral: {
     auto sle = static_cast<StructLiteralExp *>(e);
     if (sle->sd->isNested())
       return false;
@@ -540,10 +540,10 @@ bool isConstLiteral(Expression *e, bool immutableType) {
   } break;
 
   // isConst also returns 0 for string literals that are obviously constant.
-  case TOKstring:
+  case EXP::string_:
     return true;
 
-  case TOKsymoff: {
+  case EXP::symbolOffset: {
     // Note: dllimported symbols are not link-time constant.
     auto soe = static_cast<SymOffExp *>(e);
     if (VarDeclaration *vd = soe->var->isVarDeclaration()) {
@@ -640,7 +640,7 @@ void initializeArrayLiteral(IRState *p, ArrayLiteralExp *ale, LLValue *dstMem) {
 
       // try to construct it in-place
       if (!toInPlaceConstruction(&lhs, rhsExp))
-        DtoAssign(ale->loc, &lhs, toElem(rhsExp), TOKblit);
+        DtoAssign(ale->loc, &lhs, toElem(rhsExp), EXP::blit);
     }
   }
 }
@@ -828,7 +828,7 @@ void DtoCatAssignElement(const Loc &loc, DValue *array, Expression *exp) {
       gIR->ir->CreateSub(newLength, DtoConstSize_t(1), ".lastIndex");
   LLValue *lastElemPtr = DtoGEP1(ptr, lastIndex, ".lastElem");
   DLValue lastElem(arrayType->nextOf(), lastElemPtr);
-  DtoAssign(loc, &lastElem, expVal, TOKblit);
+  DtoAssign(loc, &lastElem, expVal, EXP::blit);
   callPostblit(loc, exp, lastElemPtr);
 }
 
@@ -870,13 +870,13 @@ DSliceValue *DtoCatArrays(const Loc &loc, Type *arrayType, Expression *exp1,
     do {
       arrs.push_back(DtoSlicePtr(ce->e2));
       ce = static_cast<CatExp *>(ce->e1);
-    } while (ce->op == TOKcat);
+    } while (ce->op == EXP::concatenate);
     arrs.push_back(DtoSlicePtr(ce));
 
     // Create static array from slices
     LLPointerType *ptrarraytype = isaPointer(arrs[0]);
     assert(ptrarraytype && "Expected pointer type");
-    LLStructType *arraytype = isaStruct(ptrarraytype->getElementType());
+    LLStructType *arraytype = isaStruct(ptrarraytype->getPointerElementType());
     assert(arraytype && "Expected struct type");
     LLArrayType *type = LLArrayType::get(arraytype, arrs.size());
     LLValue *array = DtoRawAlloca(type, 0, ".slicearray");
@@ -1119,7 +1119,7 @@ LLValue *DtoArrayEqCmp_memcmp(const Loc &loc, DValue *l, DValue *r,
 } // end anonymous namespace
 
 ////////////////////////////////////////////////////////////////////////////////
-LLValue *DtoArrayEquals(const Loc &loc, TOK op, DValue *l, DValue *r) {
+LLValue *DtoArrayEquals(const Loc &loc, EXP op, DValue *l, DValue *r) {
   LLValue *res = nullptr;
 
   if (r->isNull()) {
@@ -1143,7 +1143,7 @@ LLValue *DtoArrayEquals(const Loc &loc, TOK op, DValue *l, DValue *r) {
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-LLValue *DtoDynArrayIs(TOK op, DValue *l, DValue *r) {
+LLValue *DtoDynArrayIs(EXP op, DValue *l, DValue *r) {
   assert(l);
   assert(r);
 
