@@ -105,7 +105,8 @@ Statement *asmSemantic(AsmStatement *s, Scope *sc) {
   sc->func->hasReturnExp |= 8;
 
   // GCC-style asm starts with a string literal or a `(`
-  if (s->tokens->value == TOKstring || s->tokens->value == TOKlparen) {
+  if (s->tokens->value == TOK::string_ ||
+      s->tokens->value == TOK::leftParenthesis) {
     auto gas = createGccAsmStatement(s->loc, s->tokens);
     return gccAsmSemantic(gas, sc);
   }
@@ -210,7 +211,7 @@ void AsmStatement_toIR(InlineAsmStatement *stmt, IRState *irs) {
       cns = i_cns;
       break;
     case Arg_Pointer:
-      assert(arg->expr->op == TOKvar);
+      assert(arg->expr->isVarExp());
       arg_val = DtoRVal(arg->expr);
       cns = p_cns;
 
@@ -718,9 +719,10 @@ void CompoundAsmStatement_toIR(CompoundAsmStatement *stmt, IRState *p) {
 
   llvm::InlineAsm *ia = llvm::InlineAsm::get(fty, code, out_c, true);
 
-  llvm::CallInst *call = p->ir->CreateCall(
-      ia, args, retty == LLType::getVoidTy(gIR->context()) ? "" : "asm");
-  p->addInlineAsmSrcLoc(stmt->loc, call);
+  auto call = p->createInlineAsmCall(stmt->loc, ia, args);
+  if (!retty->isVoidTy()) {
+    call->setName("asm");
+  }
 
   IF_LOG Logger::cout() << "Complete asm statement: " << *call << '\n';
 
@@ -745,8 +747,7 @@ void CompoundAsmStatement_toIR(CompoundAsmStatement *stmt, IRState *p) {
     // make new blocks
     llvm::BasicBlock *bb = p->insertBB("afterasmgotoforwarder");
 
-    llvm::LoadInst *val =
-        p->ir->CreateLoad(jump_target, "__llvm_jump_target_value");
+    auto val = DtoLoad(jump_target, "__llvm_jump_target_value");
     llvm::SwitchInst *sw = p->ir->CreateSwitch(val, bb, gotoToVal.size());
 
     // add all cases
