@@ -370,8 +370,6 @@ DIType DIBuilder::CreateEnumType(TypeEnum *type) {
 }
 
 DIType DIBuilder::CreatePointerType(TypePointer *type) {
-  llvm::Type *T = DtoType(type);
-
   // TODO: The addressspace is important for dcompute targets. See e.g.
   // https://www.mail-archive.com/dwarf-discuss@lists.dwarfstd.org/msg00326.html
   const llvm::Optional<unsigned> DWARFAddressSpace = llvm::None;
@@ -380,9 +378,7 @@ DIType DIBuilder::CreatePointerType(TypePointer *type) {
 
   return DBuilder.createPointerType(
       CreateTypeDescription(type->nextOf(), /*voidToUbyte=*/true),
-      getTypeAllocSize(T) * 8, // size (bits)
-      getABITypeAlign(T) * 8,  // align (bits)
-      DWARFAddressSpace, name);
+      target.ptrsize * 8, 0, DWARFAddressSpace, name);
 }
 
 DIType DIBuilder::CreateVectorType(TypeVector *type) {
@@ -709,9 +705,6 @@ DIType DIBuilder::CreateAArrayType(TypeAArray *type) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-// new calling convention constant being proposed as a Dwarf extension
-const unsigned DW_CC_D_dmd = 0x43;
-
 DISubroutineType DIBuilder::CreateFunctionType(Type *type) {
   TypeFunction *t = type->isTypeFunction();
   assert(t);
@@ -722,12 +715,7 @@ DISubroutineType DIBuilder::CreateFunctionType(Type *type) {
   LLMetadata *params = {CreateTypeDescription(retType)};
   auto paramsArray = DBuilder.getOrCreateTypeArray(params);
 
-  // The calling convention has to be recorded to distinguish
-  // extern(D) functions from extern(C++) ones.
-  unsigned CC =
-      getIrType(t, true)->getIrFuncTy().reverseParams ? DW_CC_D_dmd : 0;
-
-  return DBuilder.createSubroutineType(paramsArray, DIFlags::FlagZero, CC);
+  return DBuilder.createSubroutineType(paramsArray, DIFlags::FlagZero, 0);
 }
 
 DISubroutineType DIBuilder::CreateEmptyFunctionType() {
@@ -775,9 +763,9 @@ DIType DIBuilder::CreateTypeDescription(Type *t, bool voidToUbyte) {
     return nullptr;
   if (t->ty == TY::Tnull) {
     // display null as void*
-    return DBuilder.createPointerType(CreateTypeDescription(Type::tvoid), 8, 8,
-                                      /* DWARFAddressSpace */ llvm::None,
-                                      "typeof(null)");
+    return DBuilder.createPointerType(
+        CreateTypeDescription(Type::tvoid), target.ptrsize * 8, 0,
+        /* DWARFAddressSpace */ llvm::None, "typeof(null)");
   }
   if (auto te = t->isTypeEnum())
     return CreateEnumType(te);
@@ -796,13 +784,11 @@ DIType DIBuilder::CreateTypeDescription(Type *t, bool voidToUbyte) {
   if (t->ty == TY::Tstruct)
     return CreateCompositeType(t);
   if (auto tc = t->isTypeClass()) {
-    LLType *T = DtoType(t);
     const auto aggregateDIType = CreateCompositeType(t);
     const auto name =
         (tc->sym->toPrettyChars(true) + llvm::StringRef("*")).str();
-    return DBuilder.createPointerType(aggregateDIType, getTypeAllocSize(T) * 8,
-                                      getABITypeAlign(T) * 8, llvm::None,
-                                      processDIName(name));
+    return DBuilder.createPointerType(aggregateDIType, target.ptrsize * 8, 0,
+                                      llvm::None, processDIName(name));
   }
   if (auto tf = t->isTypeFunction())
     return CreateFunctionType(tf);
@@ -1167,7 +1153,12 @@ void DIBuilder::EmitValue(llvm::Value *val, VarDeclaration *vd) {
 void DIBuilder::EmitLocalVariable(llvm::Value *ll, VarDeclaration *vd,
                                   Type *type, bool isThisPtr, bool forceAsLocal,
                                   bool isRefRVal,
-                                  llvm::ArrayRef<int64_t> addr) {
+#if LDC_LLVM_VER >= 1400
+                                  llvm::ArrayRef<uint64_t> addr
+#else
+                                  llvm::ArrayRef<int64_t> addr
+#endif
+                                  ) {
   if (!mustEmitFullDebugInfo())
     return;
 
