@@ -270,7 +270,7 @@ LLValue *DtoAllocaDump(LLValue *val, LLType *asType, int alignment,
 }
 
 /******************************************************************************
- * ASSERT HELPER
+ * ASSERT HELPERS
  ******************************************************************************/
 
 void DtoAssert(Module *M, const Loc &loc, DValue *msg) {
@@ -339,6 +339,21 @@ void DtoCAssert(Module *M, const Loc &loc, LLValue *msg) {
   gIR->ir->CreateUnreachable();
 }
 
+/******************************************************************************
+ * THROW HELPER
+ ******************************************************************************/
+
+void DtoThrow(const Loc &loc, DValue *e) {
+  LLFunction *fn = getRuntimeFunction(loc, gIR->module, "_d_throw_exception");
+  LLValue *arg = DtoBitCast(DtoRVal(e), fn->getFunctionType()->getParamType(0));
+
+  gIR->CreateCallOrInvoke(fn, arg);
+  gIR->ir->CreateUnreachable();
+
+  llvm::BasicBlock *bb = gIR->insertBB("afterthrow");
+  gIR->ir->SetInsertPoint(bb);
+}
+
 /******************************************************************************
  * MODULE FILE NAME
  ******************************************************************************/
@@ -369,7 +384,7 @@ void DtoGoto(const Loc &loc, LabelDsymbol *target) {
 
 // is this a good approach at all ?
 
-void DtoAssign(const Loc &loc, DValue *lhs, DValue *rhs, int op,
+void DtoAssign(const Loc &loc, DValue *lhs, DValue *rhs, EXP op,
                bool canSkipPostblit) {
   IF_LOG Logger::println("DtoAssign()");
   LOG_SCOPE;
@@ -620,8 +635,8 @@ DValue *DtoCastDelegate(const Loc &loc, DValue *val, Type *to) {
     return DtoPaintType(loc, val, to);
   }
   if (to->toBasetype()->ty == TY::Tbool) {
-    return new DImValue(to,
-                        DtoDelegateEquals(TOKnotequal, DtoRVal(val), nullptr));
+    return new DImValue(
+        to, DtoDelegateEquals(EXP::notEqual, DtoRVal(val), nullptr));
   }
   error(loc, "invalid cast from `%s` to `%s`", val->type->toChars(),
         to->toChars());
@@ -1353,7 +1368,7 @@ LLValue *makeLValue(const Loc &loc, DValue *value) {
 
   LLValue *mem = DtoAlloca(value->type, ".makelvaluetmp");
   DLValue var(value->type, mem);
-  DtoAssign(loc, &var, value, TOKblit);
+  DtoAssign(loc, &var, value, EXP::blit);
   return mem;
 }
 
@@ -1361,8 +1376,9 @@ LLValue *makeLValue(const Loc &loc, DValue *value) {
 
 void callPostblit(const Loc &loc, Expression *exp, LLValue *val) {
   Type *tb = exp->type->toBasetype();
-  if ((exp->op == TOKvar || exp->op == TOKdotvar || exp->op == TOKstar ||
-       exp->op == TOKthis || exp->op == TOKindex) &&
+  if ((exp->op == EXP::variable || exp->op == EXP::dotVariable ||
+       exp->op == EXP::star || exp->op == EXP::this_ ||
+       exp->op == EXP::index) &&
       tb->ty == TY::Tstruct) {
     StructDeclaration *sd = static_cast<TypeStruct *>(tb)->sym;
     if (sd->postblit) {
@@ -1412,19 +1428,19 @@ void AppendFunctionToLLVMGlobalCtorsDtors(llvm::Function *func,
 
 ////////////////////////////////////////////////////////////////////////////////
 
-void tokToICmpPred(TOK op, bool isUnsigned, llvm::ICmpInst::Predicate *outPred,
+void tokToICmpPred(EXP op, bool isUnsigned, llvm::ICmpInst::Predicate *outPred,
                    llvm::Value **outConst) {
   switch (op) {
-  case TOKlt:
+  case EXP::lessThan:
     *outPred = isUnsigned ? llvm::ICmpInst::ICMP_ULT : llvm::ICmpInst::ICMP_SLT;
     break;
-  case TOKle:
+  case EXP::lessOrEqual:
     *outPred = isUnsigned ? llvm::ICmpInst::ICMP_ULE : llvm::ICmpInst::ICMP_SLE;
     break;
-  case TOKgt:
+  case EXP::greaterThan:
     *outPred = isUnsigned ? llvm::ICmpInst::ICMP_UGT : llvm::ICmpInst::ICMP_SGT;
     break;
-  case TOKge:
+  case EXP::greaterOrEqual:
     *outPred = isUnsigned ? llvm::ICmpInst::ICMP_UGE : llvm::ICmpInst::ICMP_SGE;
     break;
   default:
@@ -1434,11 +1450,11 @@ void tokToICmpPred(TOK op, bool isUnsigned, llvm::ICmpInst::Predicate *outPred,
 
 ////////////////////////////////////////////////////////////////////////////////
 
-llvm::ICmpInst::Predicate eqTokToICmpPred(TOK op, bool invert) {
-  assert(op == TOKequal || op == TOKnotequal || op == TOKidentity ||
-         op == TOKnotidentity);
+llvm::ICmpInst::Predicate eqTokToICmpPred(EXP op, bool invert) {
+  assert(op == EXP::equal || op == EXP::notEqual || op == EXP::identity ||
+         op == EXP::notIdentity);
 
-  bool isEquality = (op == TOKequal || op == TOKidentity);
+  bool isEquality = (op == EXP::equal || op == EXP::identity);
   if (invert)
     isEquality = !isEquality;
 
@@ -1447,7 +1463,7 @@ llvm::ICmpInst::Predicate eqTokToICmpPred(TOK op, bool invert) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-LLValue *createIPairCmp(TOK op, LLValue *lhs1, LLValue *lhs2, LLValue *rhs1,
+LLValue *createIPairCmp(EXP op, LLValue *lhs1, LLValue *lhs2, LLValue *rhs1,
                         LLValue *rhs2) {
   const auto predicate = eqTokToICmpPred(op);
 
@@ -1591,7 +1607,7 @@ DValue *DtoSymbolAddress(const Loc &loc, Type *type, Declaration *decl) {
   if (SymbolDeclaration *sdecl = decl->isSymbolDeclaration()) {
     // this is the static initialiser (init symbol) for aggregates
     AggregateDeclaration *ad = sdecl->dsym;
-    IF_LOG Logger::print("Sym: ad=%s\n", ad->toChars());
+    IF_LOG Logger::print("init symbol of %s\n", ad->toChars());
     DtoResolveDsymbol(ad);
     auto sd = ad->isStructDeclaration();
 
@@ -1717,24 +1733,39 @@ static bool isDefaultLibSymbol(Dsymbol *sym) {
             (md->packages.length > 1 && md->packages.ptr[1] == Id::io)));
 }
 
-bool defineOnDeclare(Dsymbol* sym, bool isFunction) {
-  if (global.params.linkonceTemplates)
+bool defineOnDeclare(Dsymbol *sym, bool isFunction) {
+  // -linkonce-templates: all instantiated symbols
+  if (global.params.linkonceTemplates != LinkonceTemplates::no)
     return sym->isInstantiated();
 
-  // With -dllimport=defaultLibsOnly, an instantiated data symbol from a
-  // druntime/Phobos template may be assigned to an arbitrary binary (and culled
-  // from others via `needsCodegen()`). Define it in each referencing CU and
-  // never dllimport.
+  // -dllimport=defaultLibsOnly: all data symbols instantiated from
+  // druntime/Phobos templates
+  // see https://github.com/ldc-developers/ldc/issues/3931
   return !isFunction && global.params.dllimport == DLLImport::defaultLibsOnly &&
          sym->isInstantiated() && isDefaultLibSymbol(sym);
 }
 
 bool dllimportDataSymbol(Dsymbol *sym) {
-  return sym->isExport() || global.params.dllimport == DLLImport::all ||
-         (global.params.dllimport == DLLImport::defaultLibsOnly &&
-          // exclude instantiated symbols from druntime/Phobos templates (see
-          // `defineOnDeclare()`)
-          !sym->isInstantiated() && isDefaultLibSymbol(sym));
+  if (!global.params.targetTriple->isOSWindows())
+    return false;
+
+  if (sym->isExport() || global.params.dllimport == DLLImport::all ||
+      (global.params.dllimport == DLLImport::defaultLibsOnly &&
+       isDefaultLibSymbol(sym))) {
+    // Okay, this symbol is a candidate. Use dllimport unless we have a
+    // guaranteed-codegen'd definition in a root module.
+    if (auto mod = sym->isModule())
+      return !mod->isRoot(); // non-root ModuleInfo symbol
+    if (sym->inNonRoot())
+      return true; // not instantiated, and defined in non-root
+    if (sym->isInstantiated() && !defineOnDeclare(sym, false))
+      return true; // instantiated but potentially culled (needsCodegen())
+    if (auto vd = sym->isVarDeclaration())
+      if (vd->storage_class & STCextern)
+        return true; // externally defined global variable
+  }
+
+  return false;
 }
 
 llvm::GlobalVariable *declareGlobal(const Loc &loc, llvm::Module &module,
@@ -1752,7 +1783,7 @@ llvm::GlobalVariable *declareGlobal(const Loc &loc, llvm::Module &module,
   llvm::GlobalVariable *existing =
       module.getGlobalVariable(mangledName, /*AllowInternal=*/true);
   if (existing) {
-    const auto existingType = existing->getType()->getElementType();
+    const auto existingType = existing->getValueType();
     if (existingType != type || existing->isConstant() != isConstant ||
         existing->isThreadLocal() != isThreadLocal) {
       error(loc,
@@ -1825,11 +1856,10 @@ FuncDeclaration *getParentFunc(Dsymbol *sym) {
   // delegate) literals don't allow access to a parent context, even if they are
   // nested.
   if (FuncDeclaration *fd = sym->isFuncDeclaration()) {
-    bool certainlyNewRoot =
-        fd->isStatic() || (!fd->isThis() && fd->linkage != LINK::d) ||
-        (fd->isFuncLiteralDeclaration() &&
-         static_cast<FuncLiteralDeclaration *>(fd)->tok == TOKfunction);
-    if (certainlyNewRoot) {
+    if (auto fld = fd->isFuncLiteralDeclaration()) {
+      if (fld->tok == TOK::function_)
+        return nullptr;
+    } else if (fd->isStatic() || (!fd->isThis() && fd->linkage != LINK::d)) {
       return nullptr;
     }
   }
