@@ -63,6 +63,8 @@ LLValue *DtoAllocaDump(LLValue *val, LLType *asType, int alignment = 0,
 void DtoAssert(Module *M, const Loc &loc, DValue *msg);
 void DtoCAssert(Module *M, const Loc &loc, LLValue *msg);
 
+void DtoThrow(const Loc &loc, DValue *e);
+
 // returns module file name
 LLConstant *DtoModuleFileName(Module *M, const Loc &loc);
 
@@ -80,7 +82,7 @@ void DtoEnterMonitor(const Loc &loc, LLValue *v);
 void DtoLeaveMonitor(const Loc &loc, LLValue *v);
 
 // basic operations
-void DtoAssign(const Loc &loc, DValue *lhs, DValue *rhs, int op,
+void DtoAssign(const Loc &loc, DValue *lhs, DValue *rhs, EXP op,
                bool canSkipPostblit = false);
 
 DValue *DtoSymbolAddress(const Loc &loc, Type *type, Declaration *decl);
@@ -178,16 +180,16 @@ bool isLLVMUnsigned(Type *t);
 ///
 /// For some operations, the result can be a constant. In this case outConst is
 /// set to it, otherwise outPred is set to the predicate to use.
-void tokToICmpPred(TOK op, bool isUnsigned, llvm::ICmpInst::Predicate *outPred,
+void tokToICmpPred(EXP op, bool isUnsigned, llvm::ICmpInst::Predicate *outPred,
                    llvm::Value **outConst);
 
 /// Converts a DMD equality/identity operation token into the corresponding LLVM
 /// icmp predicate.
-llvm::ICmpInst::Predicate eqTokToICmpPred(TOK op, bool invert = false);
+llvm::ICmpInst::Predicate eqTokToICmpPred(EXP op, bool invert = false);
 
 /// For equality/identity operations, returns `(lhs1 == rhs1) & (lhs2 == rhs2)`.
 /// `(lhs1 != rhs1) | (lhs2 != rhs2)` for inequality/not-identity.
-LLValue *createIPairCmp(TOK op, LLValue *lhs1, LLValue *lhs2, LLValue *rhs1,
+LLValue *createIPairCmp(EXP op, LLValue *lhs1, LLValue *lhs2, LLValue *rhs1,
                         LLValue *rhs2);
 
 ////////////////////////////////////////////
@@ -245,8 +247,7 @@ llvm::Constant *buildStringLiteralConstant(StringExp *se, bool zeroTerm);
 /// primarily for -linkonce-templates.
 bool defineOnDeclare(Dsymbol *sym, bool isFunction);
 
-/// Indicates whether the specified data symbol is a general dllimport
-/// candidate.
+/// Indicates whether the specified data symbol is to be declared as dllimport.
 bool dllimportDataSymbol(Dsymbol *sym);
 
 /// Tries to declare an LLVM global. If a variable with the same mangled name
