@@ -83,73 +83,31 @@ void Module::checkAndAddOutputFile(const FileName &file) {
 }
 
 namespace {
-/// Ways the druntime module registry system can be implemented.
+/// Ways the druntime ModuleInfo registry system can be implemented.
 enum class RegistryStyle {
-  /// Modules are inserted into a linked list starting at the _Dmodule_ref
-  /// global.
+  /// ModuleInfo refs are inserted into a linked list starting at the
+  /// _Dmodule_ref global.
   legacyLinkedList,
 
-  /// Module references are emitted into the .minfo/__minfo section.
-  sectionSimple,
-
-  /// Module references are emitted into the __minfo section. Global
-  /// constructors/destructors make sure _d_dso_registry is invoked once per ELF
-  /// object.
-  sectionELF,
-
-  /// Module references are emitted into the .minfo section. Global
-  /// constructors/destructors make sure _d_dso_registry is invoked once per
-  /// shared object. A "TLS anchor" function to identify the TLS range
-  /// corresponding to this image is also passed to druntime.
-  sectionDarwin
+  /// Pointers to defined ModuleInfos are emitted into the special .minfo /
+  /// __minfo section. A linked binary then contains pointers to all ModuleInfos
+  /// of linked object files in that section.
+  section,
 };
 
-/// Returns the module registry style to use for the current target triple.
+/// Returns the ModuleInfo registry style to use for the current target triple.
 RegistryStyle getModuleRegistryStyle() {
   const auto &t = *global.params.targetTriple;
-
   if (t.isOSWindows() || t.getEnvironment() == llvm::Triple::Android ||
-      t.isOSBinFormatWasm()) {
-    return RegistryStyle::sectionSimple;
-  }
-
-  if (t.isOSDarwin()) {
-    return RegistryStyle::sectionDarwin;
-  }
-
-  if (t.isOSLinux() || t.isOSFreeBSD() || t.isOSNetBSD() || t.isOSOpenBSD() ||
+      t.isOSBinFormatWasm() || t.isOSDarwin() || t.isOSLinux() ||
+      t.isOSFreeBSD() || t.isOSNetBSD() || t.isOSOpenBSD() ||
       t.isOSDragonFly()) {
-    return RegistryStyle::sectionELF;
+    return RegistryStyle::section;
   }
 
   return RegistryStyle::legacyLinkedList;
 }
 
-LLGlobalVariable *declareDSOGlobal(llvm::StringRef mangledName, LLType *type,
-                                   bool isThreadLocal = false) {
-  auto global = declareGlobal(Loc(), gIR->module, type, mangledName, false,
-                              isThreadLocal, false);
-  global->setVisibility(LLGlobalValue::HiddenVisibility);
-  return global;
-}
-
-LLGlobalVariable *defineDSOGlobal(llvm::StringRef mangledName, LLConstant *init,
-                                  bool isThreadLocal = false) {
-  auto global =
-      defineGlobal(Loc(), gIR->module, mangledName, init,
-                   LLGlobalValue::LinkOnceODRLinkage, false, isThreadLocal);
-  global->setVisibility(LLGlobalValue::HiddenVisibility);
-  return global;
-}
-
-LLFunction *createDSOFunction(llvm::StringRef mangledName,
-                              LLFunctionType *type) {
-  auto fn = LLFunction::Create(type, LLGlobalValue::LinkOnceODRLinkage,
-                               mangledName, &gIR->module);
-  fn->setVisibility(LLGlobalValue::HiddenVisibility);
-  return fn;
-}
-
 /// Build ModuleReference and register function, to register the module info in
 /// the global linked list.
 ///
@@ -206,7 +164,11 @@ LLFunction *build_module_reference_and_ctor(const char *moduleMangle,
   gIR->DBuilder.EmitModuleCTor(ctor, fname.c_str());
 
   // get current beginning
-  LLValue *curbeg = builder.CreateLoad(mref, "current");
+  LLValue *curbeg = builder.CreateLoad(
+#if LDC_LLVM_VER >= 800
+      modulerefPtrTy,
+#endif
+      mref, "current");
 
   // put current beginning as the next of this one
   LLValue *gep = builder.CreateStructGEP(
@@ -222,160 +184,26 @@ LLFunction *build_module_reference_and_ctor(const char *moduleMangle,
   return ctor;
 }
 
-/// Builds a void*() function with hidden visibility that returns the address of
-/// a dummy TLS global (also with hidden visibility).
-///
-/// The global is non-zero-initialised and aligned to 16 bytes.
-llvm::Function *buildGetTLSAnchor() {
-  // Create a dummmy TLS global private to this module.
-  const auto one = llvm::ConstantInt::get(LLType::getInt8Ty(gIR->context()), 1);
-  const auto anchor =
-      defineDSOGlobal("ldc.tls_anchor", one, /*isThreadLocal=*/true);
-  anchor->setAlignment(LLMaybeAlign(16));
-
-  const auto getAnchor = createDSOFunction(
-      "ldc.get_tls_anchor", LLFunctionType::get(getVoidPtrType(), false));
-
-  IRBuilder<> builder(llvm::BasicBlock::Create(gIR->context(), "", getAnchor));
-  builder.CreateRet(anchor);
-
-  return getAnchor;
-}
-
-/// Builds the ldc.register_dso function, which is called by the global
-/// {c, d}tors to invoke _d_dso_registry.
-///
-/// Pseudocode:
-/// void ldc.register_dso() {
-///   auto record = {1, dsoSlot, minfoBeg, minfoEnd[, getTlsAnchor]};
-///   _d_dso_registry(cast(CompilerDSOData*)&record);
-/// }
-///
-/// On Darwin platforms, the record contains an extra pointer to a function
-/// which returns the address of a TLS global.
-llvm::Function *buildRegisterDSO(RegistryStyle style, llvm::Value *dsoSlot,
-                                 llvm::Value *minfoBeg, llvm::Value *minfoEnd) {
-  const auto fn = createDSOFunction(
-      "ldc.register_dso",
-      LLFunctionType::get(LLType::getVoidTy(gIR->context()), false));
-
-  const auto dsoRegistry =
-      getRuntimeFunction(Loc(), gIR->module, "_d_dso_registry");
-  const auto recordPtrTy = dsoRegistry->getFunctionType()->getContainedType(1);
-
-  llvm::Function *getTlsAnchorPtr = nullptr;
-  if (style == RegistryStyle::sectionDarwin) {
-    getTlsAnchorPtr = buildGetTLSAnchor();
-  }
-
-  {
-    const auto bb = llvm::BasicBlock::Create(gIR->context(), "", fn);
-    IRBuilder<> b(bb);
-
-    llvm::Constant *version = DtoConstSize_t(1);
-    llvm::SmallVector<llvm::Type *, 6> memberTypes;
-    memberTypes.push_back(version->getType());
-    memberTypes.push_back(dsoSlot->getType());
-    memberTypes.push_back(minfoBeg->getType());
-    memberTypes.push_back(minfoEnd->getType());
-    if (style == RegistryStyle::sectionDarwin) {
-      memberTypes.push_back(getTlsAnchorPtr->getType());
-    }
-    llvm::StructType *stype =
-        llvm::StructType::get(gIR->context(), memberTypes, false);
-    llvm::Value *record = b.CreateAlloca(stype);
-
-    unsigned i = 0;
-    b.CreateStore(version, b.CreateStructGEP(stype, record, i++));
-    b.CreateStore(dsoSlot, b.CreateStructGEP(stype, record, i++));
-    b.CreateStore(minfoBeg, b.CreateStructGEP(stype, record, i++));
-    b.CreateStore(minfoEnd, b.CreateStructGEP(stype, record, i++));
-    if (style == RegistryStyle::sectionDarwin) {
-      b.CreateStore(getTlsAnchorPtr, b.CreateStructGEP(stype, record, i++));
-    }
-
-    b.CreateCall(dsoRegistry, b.CreateBitCast(record, recordPtrTy));
-    b.CreateRetVoid();
-  }
-
-  return fn;
-}
-
-void emitModuleRefToSection(RegistryStyle style, std::string moduleMangle,
+// Emits a pointer to the specified ModuleInfo into the special
+// .minfo (COFF & MachO) / __minfo section.
+void emitModuleRefToSection(std::string moduleMangle,
                             llvm::Constant *thisModuleInfo) {
-  assert(style == RegistryStyle::sectionSimple ||
-         style == RegistryStyle::sectionELF ||
-         style == RegistryStyle::sectionDarwin);
-  // Only for the first D module to be emitted into this llvm::Module we need to
-  // create the global ctors/dtors. The magic linker symbols used to get the
-  // start and end of the .minfo section also only need to be emitted for the
-  // first D module.
-  // For all subsequent ones, we just need to emit an additional reference into
-  // the .minfo section.
-  const bool isFirst = !gIR->module.getGlobalVariable("ldc.dso_slot");
-
   const auto moduleInfoPtrTy = DtoPtrToType(getModuleInfoType());
-  const auto moduleInfoRefsSectionName =
-      global.params.targetTriple->isWindowsMSVCEnvironment()
+
+  const auto &triple = *global.params.targetTriple;
+  const auto sectionName =
+      triple.isOSBinFormatCOFF()
           ? ".minfo"
-          : style == RegistryStyle::sectionDarwin ? "__DATA,.minfo" : "__minfo";
+          : triple.isOSBinFormatMachO() ? "__DATA,.minfo" : "__minfo";
 
   const auto thismrefIRMangle =
       getIRMangledModuleRefSymbolName(moduleMangle.c_str());
-  auto thismref = defineDSOGlobal(thismrefIRMangle,
-                                  DtoBitCast(thisModuleInfo, moduleInfoPtrTy));
-  thismref->setSection(moduleInfoRefsSectionName);
+  auto thismref = defineGlobal(Loc(), gIR->module, thismrefIRMangle,
+                               DtoBitCast(thisModuleInfo, moduleInfoPtrTy),
+                               LLGlobalValue::LinkOnceODRLinkage, false, false);
+  thismref->setVisibility(LLGlobalValue::HiddenVisibility);
+  thismref->setSection(sectionName);
   gIR->usedArray.push_back(thismref);
-
-  if (!isFirst || style == RegistryStyle::sectionSimple) {
-    // Nothing left to do.
-    return;
-  }
-
-  // Use magic linker symbol names to obtain the begin and end of the .minfo
-  // section.
-  const auto magicBeginSymbolName = (style == RegistryStyle::sectionDarwin)
-                                        ? "\1section$start$__DATA$.minfo"
-                                        : "__start___minfo";
-  const auto magicEndSymbolName = (style == RegistryStyle::sectionDarwin)
-                                      ? "\1section$end$__DATA$.minfo"
-                                      : "__stop___minfo";
-  auto minfoBeg = declareDSOGlobal(magicBeginSymbolName, moduleInfoPtrTy);
-  auto minfoEnd = declareDSOGlobal(magicEndSymbolName, moduleInfoPtrTy);
-
-  // We want to have one global constructor and destructor per object (i.e.
-  // executable/shared library) that calls _d_dso_registry with the respective
-  // DSO record.
-  // To enable safe direct linking of D objects (e.g., "g++ dcode.o cppcode.o"),
-  // we emit a pair of global {c,d}tors into each object file, both pointing to
-  // a common ldc.register_dso() function.
-  // These per-object-file pairs will be folded to a single one when linking the
-  // DSO, together with the ldc.dso_slot globals and associated
-  // ldc.register_dso() functions.
-
-  // This is the DSO slot for use by the druntime implementation.
-  const auto dsoSlot =
-      defineDSOGlobal("ldc.dso_slot", getNullPtr(getVoidPtrType()));
-
-  const auto registerDSO = buildRegisterDSO(style, dsoSlot, minfoBeg, minfoEnd);
-
-  // We need to discard the {c,d}tor refs if this IR module's ldc.register_dso()
-  // function is discarded to prevent duplicate refs.
-  // Unfortunately, this doesn't work for macOS (v10.12, Xcode v9.2, LLVM
-  // v7.0.0).
-  if (style == RegistryStyle::sectionELF) {
-    llvm::appendToGlobalCtors(gIR->module, registerDSO, 65535, registerDSO);
-    llvm::appendToGlobalDtors(gIR->module, registerDSO, 65535, registerDSO);
-    return;
-  }
-
-  // macOS: emit the {c,d}tor refs manually
-  const auto dsoCtor = defineDSOGlobal("ldc.dso_ctor", registerDSO);
-  const auto dsoDtor = defineDSOGlobal("ldc.dso_dtor", registerDSO);
-  gIR->usedArray.push_back(dsoCtor);
-  gIR->usedArray.push_back(dsoDtor);
-  dsoCtor->setSection("__DATA,__mod_init_func,mod_init_funcs");
-  dsoDtor->setSection("__DATA,__mod_term_func,mod_term_funcs");
 }
 
 // Add module-private variables and functions for coverage analysis.
@@ -569,7 +397,7 @@ void registerModuleInfo(Module *m) {
     const auto miCtor = build_module_reference_and_ctor(mangle, moduleInfoSym);
     AppendFunctionToLLVMGlobalCtorsDtors(miCtor, 65535, true);
   } else {
-    emitModuleRefToSection(style, mangle, moduleInfoSym);
+    emitModuleRefToSection(mangle, moduleInfoSym);
   }
 }
 }
