@@ -12,11 +12,6 @@
 //
 //===----------------------------------------------------------------------===//
 
-#define DEBUG_TYPE "dgc2stack"
-#if LDC_LLVM_VER < 700
-#define LLVM_DEBUG DEBUG
-#endif
-
 #include "gen/attributes.h"
 #include "metadata.h"
 #include "gen/passes/Passes.h"
@@ -42,6 +37,11 @@
 #include "llvm/Support/raw_ostream.h"
 #include <algorithm>
 
+#define DEBUG_TYPE "dgc2stack"
+#if LDC_LLVM_VER < 700
+#define LLVM_DEBUG DEBUG
+#endif
+
 using namespace llvm;
 
 STATISTIC(NumGcToStack, "Number of calls promoted to constant-size allocas");
@@ -199,7 +199,7 @@ public:
     assert(ArrTy && "Dynamic array type not a struct?");
     assert(isa<IntegerType>(ArrTy->getElementType(0)));
     const PointerType *PtrTy = cast<PointerType>(ArrTy->getElementType(1));
-    Ty = PtrTy->getElementType();
+    Ty = PtrTy->getPointerElementType();
 
     // If the user explicitly disabled the limits, don't even check
     // whether the element count fits in 32 bits. This could cause
