@@ -52,7 +52,7 @@ void CompoundAsmStatement_toIR(CompoundAsmStatement *stmt, IRState *p);
 namespace {
 bool isAssertFalse(Expression *e) {
   return e ? e->type == Type::tnoreturn &&
-                 (e->op == TOKhalt || e->op == TOKassert)
+                 (e->op == EXP::halt || e->op == EXP::assert_)
            : false;
 }
 
@@ -227,14 +227,14 @@ public:
                   doPostblit = false;
             }
 
-            DtoAssign(stmt->loc, &returnValue, e, TOKblit);
+            DtoAssign(stmt->loc, &returnValue, e, EXP::blit);
             if (doPostblit)
               callPostblit(stmt->loc, stmt->exp, sretPointer);
           }
         }
       } else {
         // the return type is not void, so this is a normal "register" return
-        if (stmt->exp->op == TOKnull) {
+        if (stmt->exp->op == EXP::null_) {
           stmt->exp->type = rt;
         }
         DValue *dval = nullptr;
@@ -347,7 +347,7 @@ public:
       DValue *elem;
       // a cast(void) around the expression is allowed, but doesn't require any
       // code
-      if (e->op == TOKcast && e->type == Type::tvoid) {
+      if (e->op == EXP::cast_ && e->type == Type::tvoid) {
         elem = toElemDtor(static_cast<CastExp *>(e)->e1);
       } else {
         elem = toElemDtor(e);
@@ -934,19 +934,7 @@ public:
     emitCoverageLinecountInc(stmt->loc);
 
     assert(stmt->exp);
-    DValue *e = toElemDtor(stmt->exp);
-
-    llvm::Function *fn =
-        getRuntimeFunction(stmt->loc, irs->module, "_d_throw_exception");
-    LLValue *arg =
-        DtoBitCast(DtoRVal(e), fn->getFunctionType()->getParamType(0));
-
-    irs->CreateCallOrInvoke(fn, arg);
-    irs->ir->CreateUnreachable();
-
-    // TODO: Should not be needed.
-    llvm::BasicBlock *bb = irs->insertBB("afterthrow");
-    irs->ir->SetInsertPoint(bb);
+    DtoThrow(stmt->loc, toElemDtor(stmt->exp));
   }
 
   //////////////////////////////////////////////////////////////////////////
@@ -1340,7 +1328,7 @@ public:
       }
     }
 
-    if (stmt->op == TOKforeach) {
+    if (stmt->op == TOK::foreach_) {
       new llvm::StoreInst(zerokey, keyvar, irs->scopebb());
     } else {
       new llvm::StoreInst(niters, keyvar, irs->scopebb());
@@ -1358,9 +1346,9 @@ public:
 
     LLValue *done = nullptr;
     LLValue *load = DtoLoad(keyvar);
-    if (stmt->op == TOKforeach) {
+    if (stmt->op == TOK::foreach_) {
       done = irs->ir->CreateICmpULT(load, niters);
-    } else if (stmt->op == TOKforeach_reverse) {
+    } else if (stmt->op == TOK::foreach_reverse_) {
       done = irs->ir->CreateICmpUGT(load, zerokey);
       load = irs->ir->CreateSub(load, LLConstantInt::get(keytype, 1, false));
       DtoStore(load, keyvar);
@@ -1377,14 +1365,14 @@ public:
     PGO.emitCounterIncrement(stmt);
 
     // get value for this iteration
-    LLValue *loadedKey = irs->ir->CreateLoad(keyvar);
+    LLValue *loadedKey = DtoLoad(keyvar);
     LLValue *gep = DtoGEP1(val, loadedKey);
 
     if (!stmt->value->isRef() && !stmt->value->isOut()) {
       // Copy value to local variable, and use it as the value variable.
       DLValue dst(stmt->value->type, valvar);
       DLValue src(stmt->value->type, gep);
-      DtoAssign(stmt->loc, &dst, &src, TOKassign);
+      DtoAssign(stmt->loc, &dst, &src, EXP::assign);
       getIrLocal(stmt->value)->value = valvar;
     } else {
       // Use the GEP as the address of the value variable.
@@ -1404,7 +1392,7 @@ public:
 
     // next
     irs->ir->SetInsertPoint(nextbb);
-    if (stmt->op == TOKforeach) {
+    if (stmt->op == TOK::foreach_) {
       LLValue *load = DtoLoad(keyvar);
       load = irs->ir->CreateAdd(load, LLConstantInt::get(keytype, 1, false));
       DtoStore(load, keyvar);
@@ -1442,7 +1430,7 @@ public:
     LLValue *keyval = DtoRawVarDeclaration(stmt->key);
 
     // store initial value in key
-    if (stmt->op == TOKforeach) {
+    if (stmt->op == TOK::foreach_) {
       DtoStore(lower, keyval);
     } else {
       DtoStore(upper, keyval);
@@ -1465,11 +1453,11 @@ public:
     assert(lower->getType() == upper->getType());
     llvm::ICmpInst::Predicate cmpop;
     if (isLLVMUnsigned(stmt->key->type)) {
-      cmpop = (stmt->op == TOKforeach) ? llvm::ICmpInst::ICMP_ULT
-                                       : llvm::ICmpInst::ICMP_UGT;
+      cmpop = (stmt->op == TOK::foreach_) ? llvm::ICmpInst::ICMP_ULT
+                                          : llvm::ICmpInst::ICMP_UGT;
     } else {
-      cmpop = (stmt->op == TOKforeach) ? llvm::ICmpInst::ICMP_SLT
-                                       : llvm::ICmpInst::ICMP_SGT;
+      cmpop = (stmt->op == TOK::foreach_) ? llvm::ICmpInst::ICMP_SLT
+                                          : llvm::ICmpInst::ICMP_SGT;
     }
     LLValue *cond = irs->ir->CreateICmp(cmpop, lower, upper);
 
@@ -1486,7 +1474,7 @@ public:
     PGO.emitCounterIncrement(stmt);
 
     // reverse foreach decrements here
-    if (stmt->op == TOKforeach_reverse) {
+    if (stmt->op == TOK::foreach_reverse_) {
       LLValue *v = DtoLoad(keyval);
       LLValue *one = LLConstantInt::get(v->getType(), 1, false);
       v = irs->ir->CreateSub(v, one);
@@ -1509,7 +1497,7 @@ public:
     irs->ir->SetInsertPoint(nextbb);
 
     // forward foreach increments here
-    if (stmt->op == TOKforeach) {
+    if (stmt->op == TOK::foreach_) {
       LLValue *v = DtoLoad(keyval);
       LLValue *one = LLConstantInt::get(v->getType(), 1, false);
       v = irs->ir->CreateAdd(v, one);
