@@ -98,7 +98,7 @@ LLFunctionType *DtoExtractFunctionType(LLType *type) {
     return fty;
   }
   if (LLPointerType *pty = isaPointer(type)) {
-    if (LLFunctionType *fty = isaFunction(pty->getElementType())) {
+    if (LLFunctionType *fty = isaFunction(pty->getPointerElementType())) {
       return fty;
     }
   }
@@ -131,7 +131,11 @@ static void addExplicitArguments(std::vector<LLValue *> &args, AttrSet &attrs,
     Type *argType = argexps[i]->type;
     bool passByVal = gABI->passByVal(irFty.type, argType);
 
+#if LDC_LLVM_VER >= 1400
+    llvm::AttrBuilder initialAttrs(getGlobalContext());
+#else
     llvm::AttrBuilder initialAttrs;
+#endif
     if (passByVal) {
 #if LDC_LLVM_VER >= 1200
       initialAttrs.addByValAttr(DtoType(argType));
@@ -144,7 +148,7 @@ static void addExplicitArguments(std::vector<LLValue *> &args, AttrSet &attrs,
       DtoAddExtendAttr(argType, initialAttrs);
     }
 
-    optionalIrArgs.push_back(new IrFuncTyArg(argType, passByVal, initialAttrs));
+    optionalIrArgs.push_back(new IrFuncTyArg(argType, passByVal, std::move(initialAttrs)));
     optionalIrArgs.back()->parametersIdx = i;
   }
 
@@ -155,8 +159,6 @@ static void addExplicitArguments(std::vector<LLValue *> &args, AttrSet &attrs,
   args.resize(implicitLLArgCount + explicitLLArgCount,
               static_cast<llvm::Value *>(nullptr));
 
-  // Iterate the explicit arguments from left to right in the D source,
-  // which is the reverse of the LLVM order if irFty.reverseParams is true.
   size_t dArgIndex = 0;
   for (size_t i = 0; i < explicitLLArgCount; ++i, ++dArgIndex) {
     const bool isVararg = (i >= formalLLArgCount);
@@ -199,9 +201,7 @@ static void addExplicitArguments(std::vector<LLValue *> &args, AttrSet &attrs,
     llvm::Value *llVal = irFty.putArg(*irArg, dval, isLValueExp,
                                       dArgIndex == explicitDArgCount - 1);
 
-    const size_t llArgIdx =
-        implicitLLArgCount +
-        (irFty.reverseParams ? explicitLLArgCount - i - 1 : i);
+    const size_t llArgIdx = implicitLLArgCount + i;
     llvm::Type *const paramType =
         (isVararg ? nullptr : calleeType->getParamType(llArgIdx));
 
@@ -307,8 +307,7 @@ static LLType *getPtrToAtomicType(LLType *type) {
 bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
                             DValue *&result) {
   // va_start instruction
-  if (fndecl->llvmInternal == LLVMva_start ||
-      fndecl->ident == Id::builtin_va_start) {
+  if (fndecl->llvmInternal == LLVMva_start) {
     if (e->arguments->length < 1 || e->arguments->length > 2) {
       e->error("`va_start` instruction expects 1 (or 2) arguments");
       fatal();
@@ -327,8 +326,7 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
   }
 
   // va_copy instruction
-  if (fndecl->llvmInternal == LLVMva_copy ||
-      fndecl->ident == Id::builtin_va_copy) {
+  if (fndecl->llvmInternal == LLVMva_copy) {
     if (e->arguments->length != 2) {
       e->error("`va_copy` instruction expects 2 arguments");
       fatal();
@@ -360,8 +358,7 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
   }
 
   // va_end instruction
-  if (fndecl->llvmInternal == LLVMva_end ||
-      fndecl->ident == Id::builtin_va_end) {
+  if (fndecl->llvmInternal == LLVMva_end) {
     if (e->arguments->length != 1) {
       e->error("`va_end` instruction expects 1 argument");
       fatal();
@@ -454,7 +451,7 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
     int atomicOrdering = (*e->arguments)[1]->toInteger();
 
     LLValue *ptr = DtoRVal(exp);
-    LLType *pointeeType = ptr->getType()->getContainedType(0);
+    LLType *pointeeType = getPointeeType(ptr);
     Type *retType = exp->type->nextOf();
 
     if (!pointeeType->isIntegerTy()) {
@@ -468,13 +465,18 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
       }
     }
 
-    llvm::LoadInst *load = p->ir->CreateLoad(ptr);
-    if (auto alignment = getTypeAllocSize(load->getType())) {
+    const auto loadedType = getPointeeType(ptr);
+    llvm::LoadInst *load = p->ir->CreateLoad(
+#if LDC_LLVM_VER >= 800
+        loadedType,
+#endif
+        ptr);
+    if (auto alignment = getTypeAllocSize(loadedType)) {
       load->setAlignment(LLAlign(alignment));
     }
     load->setAtomic(llvm::AtomicOrdering(atomicOrdering));
     llvm::Value *val = load;
-    if (val->getType() != pointeeType) {
+    if (loadedType != pointeeType) {
       val = DtoAllocaDump(val, retType);
       result = new DLValue(retType, val);
     } else {
@@ -701,6 +703,8 @@ public:
     addArguments();
   }
 
+  bool hasContext = false; // set after addImplicitArgs invocation
+
 private:
   // passed:
   std::vector<LLValue *> &args;
@@ -744,12 +748,13 @@ private:
            "Sret arg not sret or inreg?");
   }
 
-  // Adds an optional context/this pointer argument.
+  // Adds an optional context/this pointer argument and sets hasContext.
   void addContext() {
-    bool thiscall = irFty.arg_this;
-    bool nestedcall = irFty.arg_nest;
+    const bool thiscall = irFty.arg_this;
+    const bool nestedcall = irFty.arg_nest;
 
-    if (!thiscall && !isDelegateCall && !nestedcall)
+    hasContext = thiscall || nestedcall || isDelegateCall;
+    if (!hasContext)
       return;
 
     size_t index = args.size();
@@ -864,9 +869,6 @@ DValue *DtoCallFunction(const Loc &loc, Type *resulttype, DValue *fnval,
       DtoExtractFunctionType(callable->getType());
   assert(callableTy);
 
-  const auto callconv =
-      gABI->callingConv(tf->linkage, tf, dfnval ? dfnval->func : nullptr);
-
   //     IF_LOG Logger::cout() << "callable: " << *callable << '\n';
 
   // parameter attributes
@@ -1038,28 +1040,26 @@ DValue *DtoCallFunction(const Loc &loc, Type *resulttype, DValue *fnval,
 
   // set calling convention and parameter attributes
   LLAttributeList &attrlist = attrs;
-  if (dfnval && dfnval->func) {
-    LLFunction *llfunc = llvm::dyn_cast<LLFunction>(DtoRVal(dfnval));
-    if (llfunc && llfunc->isIntrinsic()) // override intrinsic attrs
-    {
-      attrlist = llvm::Intrinsic::getAttributes(
-          gIR->context(),
-          static_cast<llvm::Intrinsic::ID>(llfunc->getIntrinsicID()));
-    } else {
-      call->setCallingConv(callconv);
+  if (auto cf = call->getCalledFunction()) {
+    call->setCallingConv(cf->getCallingConv());
+    if (cf->isIntrinsic()) { // override intrinsic attrs
+      attrlist =
+          llvm::Intrinsic::getAttributes(gIR->context(), cf->getIntrinsicID());
     }
+  } else if (dfnval) {
+    call->setCallingConv(gABI->callingConv(dfnval->func));
   } else {
-    call->setCallingConv(callconv);
+    call->setCallingConv(gABI->callingConv(tf, iab.hasContext));
   }
   // merge in function attributes set in callOrInvoke
 #if LDC_LLVM_VER >= 1400
+  auto attrbuildattribs = call->getAttributes().getFnAttrs();
   attrlist = attrlist.addFnAttributes(
-    gIR->context(),
-    llvm::AttrBuilder(call->getAttributes(), LLAttributeList::FunctionIndex));
+      gIR->context(), llvm::AttrBuilder(gIR->context(), attrbuildattribs));
 #else
   attrlist = attrlist.addAttributes(
-    gIR->context(), LLAttributeList::FunctionIndex,
-    llvm::AttrBuilder(call->getAttributes(), LLAttributeList::FunctionIndex));
+      gIR->context(), LLAttributeList::FunctionIndex,
+      llvm::AttrBuilder(call->getAttributes(), LLAttributeList::FunctionIndex));
 #endif
   call->setAttributes(attrlist);
 
