@@ -258,7 +258,7 @@ public:
 
     // string literal to dyn array:
     // reinterpret the string data as an array, calculate the length
-    if (e->e1->op == TOKstring && tb->ty == TY::Tarray) {
+    if (e->e1->op == EXP::string_ && tb->ty == TY::Tarray) {
 #if 0
             StringExp *strexp = static_cast<StringExp*>(e1);
             size_t datalen = strexp->sz * strexp->len;
@@ -278,7 +278,7 @@ public:
       result = llvm::ConstantExpr::getBitCast(toConstElem(e->e1, p), lltype);
     }
     // global variable to pointer
-    else if (tb->ty == TY::Tpointer && e->e1->op == TOKvar) {
+    else if (tb->ty == TY::Tpointer && e->e1->op == EXP::variable) {
       VarDeclaration *vd =
           static_cast<VarExp *>(e->e1)->var->isVarDeclaration();
       assert(vd);
@@ -294,7 +294,7 @@ public:
       }
       result = DtoBitCast(value, DtoType(tb));
     } else if (tb->ty == TY::Tclass && e->e1->type->ty == TY::Tclass &&
-               e->e1->op == TOKclassreference) {
+               e->e1->op == EXP::classReference) {
       auto cd = static_cast<ClassReferenceExp *>(e->e1)->originalClass();
       llvm::Constant *instance = toConstElem(e->e1, p);
       if (InterfaceDeclaration *it =
@@ -352,9 +352,9 @@ public:
       } else {
         // Offset isn't a multiple of base type size, just cast to i8* and
         // apply the byte offset.
-        auto voidPtrType = getVoidPtrType();
+        auto i8 = LLType::getInt8Ty(gIR->context());
         result = llvm::ConstantExpr::getGetElementPtr(
-            voidPtrType->getElementType(), DtoBitCast(base, voidPtrType),
+            i8, DtoBitCast(base, i8->getPointerTo()),
             DtoConstSize_t(e->offset));
       }
     }
@@ -428,7 +428,7 @@ public:
       return;
     }
 
-    if (e->e1->op == TOKslice || e->e1->op == TOKdotvar) {
+    if (e->e1->op == EXP::slice || e->e1->op == EXP::dotVariable) {
       visit(static_cast<Expression *>(e));
       return;
     }
@@ -446,17 +446,17 @@ public:
     FuncLiteralDeclaration *fd = e->fd;
     assert(fd);
 
-    if (fd->tok == TOKreserved && e->type->ty == TY::Tpointer) {
+    if (fd->tok == TOK::reserved && e->type->ty == TY::Tpointer) {
       // This is a lambda that was inferred to be a function literal instead
       // of a delegate, so set tok here in order to get correct types/mangling.
       // Horrible hack, but DMD does the same thing in FuncExp::toElem and
       // other random places.
-      fd->tok = TOKfunction;
+      fd->tok = TOK::function_;
       fd->vthis = nullptr;
     }
 
-    if (fd->tok != TOKfunction) {
-      assert(fd->tok == TOKdelegate || fd->tok == TOKreserved);
+    if (fd->tok != TOK::function_) {
+      assert(fd->tok == TOK::delegate_ || fd->tok == TOK::reserved);
 
       // Only if the function doesn't access its nested context, we can emit a
       // constant delegate with context pointer being null.
@@ -477,7 +477,7 @@ public:
     result = DtoCallee(fd, false);
     assert(result);
 
-    if (fd->tok != TOKfunction) {
+    if (fd->tok != TOK::function_) {
       auto contextPtr = getNullPtr(getVoidPtrType());
       result = LLConstantStruct::getAnon(gIR->context(), {contextPtr, result});
     }
@@ -548,11 +548,15 @@ public:
       const size_t nexprs = e->elements->length;
       for (size_t i = 0; i < nexprs; i++) {
         if (auto elem = (*e->elements)[i]) {
-          LLConstant *c = toConstElem(elem, p);
-          // extend i1 to i8
-          if (c->getType() == LLType::getInt1Ty(p->context()))
-            c = llvm::ConstantExpr::getZExt(c, LLType::getInt8Ty(p->context()));
-          varInits[e->sd->fields[i]] = c;
+          if (!elem->isVoidInitExp()) {
+            LLConstant *c = toConstElem(elem, p);
+            // extend i1 to i8
+            if (c->getType() == LLType::getInt1Ty(p->context())) {
+              c = llvm::ConstantExpr::getZExt(c,
+                                              LLType::getInt8Ty(p->context()));
+            }
+            varInits[e->sd->fields[i]] = c;
+          }
         }
       }
 
@@ -599,11 +603,13 @@ public:
           classHierachy.pop();
           for (size_t j = 0; j < cur->fields.length; ++j) {
             if (auto elem = (*value->elements)[i]) {
-              VarDeclaration *field = cur->fields[j];
-              IF_LOG Logger::println("Getting initializer for: %s",
-                                     field->toChars());
-              LOG_SCOPE;
-              varInits[field] = toConstElem(elem, p);
+              if (!elem->isVoidInitExp()) {
+                VarDeclaration *field = cur->fields[j];
+                IF_LOG Logger::println("Getting initializer for: %s",
+                                       field->toChars());
+                LOG_SCOPE;
+                varInits[field] = toConstElem(elem, p);
+              }
             }
             ++i;
           }
