@@ -66,13 +66,9 @@ StructLiteralExp *getLdcAttributesStruct(Expression *attr) {
     return nullptr;
   }
 
-  if (e->op != TOKstructliteral) {
-    return nullptr;
-  }
-
-  auto sle = static_cast<StructLiteralExp *>(e);
-  if (isFromMagicModule(sle, Id::attributes)) {
-    return sle;
+  if (auto sle = e->isStructLiteralExp()) {
+    if (isFromMagicModule(sle, Id::attributes))
+      return sle;
   }
 
   return nullptr;
@@ -109,16 +105,10 @@ StructLiteralExp *getMagicAttribute(Dsymbol *sym, const Identifier *id,
   // Loop over all UDAs and early return the expression if a match was found.
   Expressions *attrs = sym->userAttribDecl->getAttributes();
   expandTuples(attrs);
-  for (auto &attr : *attrs) {
-    if (attr->op != TOKstructliteral)
-      continue;
-    auto sle = static_cast<StructLiteralExp *>(attr);
-    if (!isFromMagicModule(sle, from))
-      continue;
-
-    if (id == sle->sd->ident) {
-      return sle;
-    }
+  for (auto attr : *attrs) {
+    if (auto sle = attr->isStructLiteralExp())
+      if (isFromMagicModule(sle, from) && id == sle->sd->ident)
+        return sle;
   }
 
   return nullptr;
@@ -136,17 +126,10 @@ void callForEachMagicAttribute(Dsymbol &sym, const Identifier *id,
   // Loop over all UDAs and call `action` if a match was found.
   Expressions *attrs = sym.userAttribDecl->getAttributes();
   expandTuples(attrs);
-  for (auto &attr : *attrs) {
-    if (attr->op != TOKstructliteral)
-      continue;
-    auto sle = static_cast<StructLiteralExp *>(attr);
-    if (!isFromMagicModule(sle, from))
-      continue;
-    if (id != sle->sd->ident)
-      continue;
-
-    // Match found!
-    action(sle);
+  for (auto attr : *attrs) {
+    if (auto sle = attr->isStructLiteralExp())
+      if (isFromMagicModule(sle, from) && id == sle->sd->ident)
+        action(sle);
   }
 }
 
@@ -208,14 +191,14 @@ void applyAttrAllocSize(StructLiteralExp *sle, IrFunction *irFunc) {
   unsigned offset = llvmNumParams - numUserParams;
 
   // Calculate the param indices for the function as defined in LLVM IR
-  auto llvmSizeIdx =
-      irFunc->irFty.reverseParams ? numUserParams - sizeArgIdx - 1 : sizeArgIdx;
-  auto llvmNumIdx =
-      irFunc->irFty.reverseParams ? numUserParams - numArgIdx - 1 : numArgIdx;
-  llvmSizeIdx += offset;
-  llvmNumIdx += offset;
+  const auto llvmSizeIdx = sizeArgIdx + offset;
+  const auto llvmNumIdx = numArgIdx + offset;
 
+#if LDC_LLVM_VER >= 1400
+  llvm::AttrBuilder builder(getGlobalContext());
+#else
   llvm::AttrBuilder builder;
+#endif
   if (numArgIdx >= 0) {
     builder.addAllocSizeAttr(llvmSizeIdx, llvmNumIdx);
   } else {
@@ -446,7 +429,11 @@ void applyFuncDeclUDAs(FuncDeclaration *decl, IrFunction *irFunc) {
       if (ident == Id::udaAllocSize) {
         applyAttrAllocSize(sle, irFunc);
       } else if (ident == Id::udaLLVMAttr) {
+#if LDC_LLVM_VER >= 1400
+        llvm::AttrBuilder attrs(getGlobalContext());
+#else
         llvm::AttrBuilder attrs;
+#endif
         applyAttrLLVMAttr(sle, attrs);
 #if LDC_LLVM_VER >= 1400
         func->addFnAttrs(attrs);
@@ -466,8 +453,9 @@ void applyFuncDeclUDAs(FuncDeclaration *decl, IrFunction *irFunc) {
         applyAttrTarget(sle, func, irFunc);
       } else if (ident == Id::udaAssumeUsed) {
         applyAttrAssumeUsed(*gIR, sle, func);
-      } else if (ident == Id::udaWeak || ident == Id::udaKernel) {
-        // @weak and @kernel are applied elsewhere
+      } else if (ident == Id::udaWeak || ident == Id::udaKernel ||
+                 ident == Id::udaNoSanitize) {
+        // These UDAs are applied elsewhere, thus should silently be ignored here.
       } else if (ident == Id::udaDynamicCompile) {
         irFunc->dynamicCompile = true;
       } else if (ident == Id::udaDynamicCompileEmit) {
