@@ -1,7 +1,6 @@
 // check the expression parser
-/* TEST_OUTPUT
+/* TEST_OUTPUT:
 ---
-fail_compilation/failcstuff1.c(51): Error: attributes should be specified before the function definition
 fail_compilation/failcstuff1.c(100): Error: no members for `enum E21962`
 fail_compilation/failcstuff1.c(101): Error: no members for anonymous enum
 fail_compilation/failcstuff1.c(152): Error: `;` or `,` expected
@@ -160,5 +159,3 @@ fail_compilation/failcstuff1.c(551): Error: missing comma or semicolon after dec
 #line 550
 
 int * pluto p;
-
-
